package com.bocloud.fms.booter.config;

import com.megatron.common.utils.MapTools;
import org.springframework.amqp.core.Binding;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.DirectExchange;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.Map;

/**
 * rabbitmq配置类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月30日
 */
@Configuration
@ConfigurationProperties(prefix = "spring.rabbitmq")
public class RabbitMQConfiguration {

    private final Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");
    private final ConnectionFactory connectionFactory;

    @Autowired
    public RabbitMQConfiguration(ConnectionFactory connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    @Bean
    public DirectExchange defaultExchange() {
        return new DirectExchange("bocloud.direct.exchange", true, true);
    }

    @Bean
    public Queue declareFlowExecuteResultQueue() {
        return new Queue("flow.execute.result.queue", true, false, false, arguments);
    }

    @Bean
    public Binding bindingFlowExecuteResultQueue() {
        return BindingBuilder.bind(declareFlowExecuteResultQueue()).to(defaultExchange()).withQueueName();
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate amqpTemplate() {
        RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
        amqpTemplate.setMessageConverter(new Jackson2JsonMessageConverter());
        amqpTemplate.setExchange(defaultExchange().getName());
        return amqpTemplate;
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitAdmin rabbitAdmin() {
        return new RabbitAdmin(connectionFactory);
    }
}
