package com.bocloud.fms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.fms.intf.FlowInstanceService;
import com.megatron.common.utils.Common;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.MessageListener;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 接收流程通过后资源实施结果
 *
 * @author zhangyf
 * @since 2019年7月26日
 */
@Slf4j
@Component
public class ExecuteResultListener implements MessageListener {

    @Autowired
    FlowInstanceService flowInstanceService;

    /**
     * @param message <p>
     *                {
     *                "id",1,
     *                "result":"SUCCESS|FAILED"
     *                }
     *                </p>
     */
    @RabbitListener(queues = "flow.execute.result.queue", admin = "rabbitAdmin", autoStartup = "true")
    @Override
    public void onMessage(Message message) {
        try {
            String content = new String(message.getBody());
            JSONObject jsonObject = JSONObject.parseObject(content);
            FlowInstance flowInstance = flowInstanceService.getFlowInstance(jsonObject.getLong(Common.ID));
            flowInstance.setExecuteResult(jsonObject.getString("result"));
            flowInstanceService.updateFlowInstance(flowInstance);
        } catch (Exception e) {
            log.error("receive execute result after adopt audited error：{}", e.getMessage(), e);
        }
    }
}
