package com.bocloud.fms.repository;

import com.megatron.framework.core.CurrentService;


import com.bocloud.fms.entity.FlowDefinition;
import com.google.common.collect.Maps;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.ListTool;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.*;

/**
 * @author hufy
 */
@Repository
public class FlowDefinitionRepository extends BasicGenericDao<FlowDefinition, Long> {

    public FlowDefinitionRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 保存流程定义
     * @param definition 流程定义实体类
     */
    public void saveDefinition(FlowDefinition definition) {
        definition.setGmtModify(new Date());
        super.getEngine().save(definition);
        // 如果baseline_id为空，则baseline是自己
        if (definition.getBaseLineId() == null) {
            definition.setBaseLineId(definition.getId());
            this.update(definition);
        }
    }

    /**
     * @param pager 分页对象
     * @return 流程定义集合
     */
    public int count(Pager pager, boolean latest) {
        String sql = "SELECT count(*) FROM wf_definition wf where is_deleted = 0 ";
        if (latest) {
            sql = "SELECT count(*) FROM wf_definition wf "
                    + "JOIN (SELECT max(version) AS version,baseline_id FROM wf_definition GROUP BY baseline_id) wf2 "
                    + "ON wf.baseline_id = wf2.baseline_id AND wf.version = wf2.version " + "where is_deleted = 0 ";
        }
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        sql = getQueryBuilder().buildRaw(sql, pager.getParams(), null, "wf");
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * @param pager 分页对象
     * @return 流程定义集合
     */
    public List<FlowDefinition> getDefinitions(Pager pager, boolean latest) {
        String sql = "SELECT wf.* FROM wf_definition wf where is_deleted = 0 ";
        if (latest) {
            sql = "SELECT wf.* FROM wf_definition wf "
                    + "JOIN (SELECT max(version) AS version,baseline_id FROM wf_definition GROUP BY baseline_id) wf2 "
                    + "ON wf.baseline_id = wf2.baseline_id AND wf.version = wf2.version " + "where is_deleted = 0 ";
        }
        Map<String, String> sorter = Maps.newHashMap();
        sorter.put("gmtCreate", Common.ONE);
        pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(sorter));
        sql = getQueryBuilder().buildRaw(sql, pager, "wf");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        return super.list(FlowDefinition.class, sql, paramMap);
    }

    /**
     * 获取最新定义的版本
     * @param id id
     * @return 流程定义实体类
     */
    public FlowDefinition getLatestDefinition(Long id) {
        Map<String, Object> params = new HashMap<>(16);
        StringBuilder query = new StringBuilder("SELECT wf.* FROM wf_definition wf "
                + "JOIN (SELECT wf1.baseline_id, wf1.version FROM ( SELECT MAX(VERSION) AS version, baseline_id FROM wf_definition GROUP BY baseline_id ) wf1 "
                + "JOIN ( SELECT * FROM wf_definition WHERE id = :id ");
        params.put("id", id);
        query.append(") wf2 ").append("ON wf1.baseline_id = wf2.baseline_id) wf3 ")
                .append("ON wf.baseline_id = wf3.baseline_id AND wf.version = wf3.version");
        List<FlowDefinition> objects = this.list(FlowDefinition.class, query.toString(), params);
        if (!ListTool.isEmpty(objects)) {
            return (objects.get(0));
        } else {
            return null;
        }
    }

    /**
     * 通过id获取流程定义
     * @param id id
     * @return 流程实体类
     */
    public FlowDefinition getDefinition(Long id) {
        return super.get(FlowDefinition.class, id);
    }

    /**
     * 更改名称
     * @param baseLineId 流程定义id
     * @param name 更改后的名称
     */
    public void updateName(Long baseLineId, String name) {
        String sql = "update wf_definition set name = :name where baseline_id = :baseLineId ";
        Map<String, Object> paramMap = MapTools.simpleMap("name", name);
        paramMap.put("baseLineId", baseLineId);
        super.execute(sql, paramMap);
    }

}
