package com.bocloud.fms.repository;

import com.megatron.framework.core.CurrentService;


import com.bocloud.fms.entity.FlowInstance;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * 流程工作单实体类（一般称为流程实例）
 *
 * @author weiwei
 */
@Repository("flowInstanceRepository")
public class FlowInstanceRepository extends BasicGenericDao<FlowInstance, Long> {

    public FlowInstanceRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 通过id获取实例
     * @param id id
     * @return 流程实例
     */
    public FlowInstance query(Long id) {
        String sql = "select a.* from wf_instance a where a.is_deleted = 0 and a.id = :id";
        Map<String, Object> params = MapTools.simpleMap(Common.ID, id);
        List<FlowInstance> list = this.list(FlowInstance.class, sql, params);
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 通过分页参数获取数量
     * @param pager 分页参数
     * @return 数量
     */
    public Long count(Pager pager) {
        List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
        boolean ifQueryOperator = params.stream().anyMatch(param -> param.getParam().containsKey("operatorId"));
        boolean ifQueryActor = params.stream().anyMatch(param -> param.getParam().containsKey("actorId"));

        StringBuilder sql = new StringBuilder("select count(distinct o.id) from wf_instance o ");
        sql.append(" left join wf_definition p on p.id = o.def_id ");
        if (ifQueryOperator) {
            // 排除起点的数据，fix bug PBSM-6956 2019年9月10日 13:58:55
            sql.append(" left join wf_node wt on o.id = wt.instance_id and (wt.code != 'DRAFT' or wt.code is null) ");
        }
        if (ifQueryActor) {
            if (ifQueryOperator) {
                sql.append(" left join wf_node_actor wta on wta.node_id = wt.id ");
            } else {
                sql.append(
                        "  left join wf_node wt on o.id = wt.instance_id left join wf_node_actor wta on wta.node_id = wt.id ");
            }
        }
        sql.append(" where 1=1 ");
        String querySql = getQueryBuilder().buildRaw(sql.toString(), pager.getParams(), null, null);
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        return super.countQuery(querySql, paramMap);
    }

    /**
     * 获取流程实例列表
     * @param pager 分页参数
     * @return 列表数据
     */
    public List<FlowInstance> getFlowInstances(Pager pager) {
        List<Param> params = Optional.ofNullable(pager.getParams()).orElse(new ArrayList<>());
        boolean ifQueryOperator = params.stream().anyMatch(param -> param.getParam().containsKey("operatorId"));
        boolean ifQueryActor = params.stream().anyMatch(param -> param.getParam().containsKey("actorId"));

        StringBuilder sql = new StringBuilder("select  o.* from wf_instance o ");
        sql.append(" left join wf_definition p on p.id = o.def_id ");
        if (ifQueryOperator) {
            // 排除起点的数据，fix bug PBSM-6956 2019年9月10日 13:58:55
            sql.append(" left join wf_node wt on o.id = wt.instance_id and (wt.code != 'DRAFT' or wt.code is null) ");
        }
        if (ifQueryActor) {
            if (ifQueryOperator) {
                sql.append(" left join wf_node_actor wta on wta.node_id = wt.id ");
            } else {
                sql.append(
                        "  left join wf_node wt on o.id = wt.instance_id left join wf_node_actor wta on wta.node_id = wt.id ");
            }
        }
        sql.append(" where 1=1 ");
        String querySql = getQueryBuilder().buildRaw(sql.toString(), pager, "");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        return super.list(FlowInstance.class, querySql, paramMap);
    }


}
