package com.bocloud.fms.repository;

import com.megatron.framework.core.CurrentService;



import com.bocloud.fms.entity.FlowNode;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.BasicGenericDao;
import com.megatron.database.utils.QueryBuilder;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

/**
 * @author hufy
 */
@Repository
public class FlowNodeRepository extends BasicGenericDao<FlowNode, Long> {

    public FlowNodeRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    public FlowNode getNode(Long taskId) {
        return this.get(FlowNode.class, taskId);
    }

    /**
     * 获取下一节点信息
     * @param flowInstanceId 流程实例id
     * @param nodeId 当前节点id
     * @param parentTaskId 父任务id
     * @return 后面节点信息
     */
    public List<FlowNode> getNextActiveNodes(Long flowInstanceId, String nodeId, Long parentTaskId) {
        String sql = "select * from wf_node where instance_id=:instanceId and node_id=:nodeId and parent_node_id=:parentTaskId";
        Map<String, Object> paramMap = MapTools.simpleMap("instanceId", flowInstanceId);
        paramMap.put("nodeId", nodeId);
        paramMap.put("parentTaskId", parentTaskId);
        return this.list(FlowNode.class, sql, paramMap);
    }

    /**
     * 获取节点
     * @param pager 分页参数
     * @return 节点list
     */
    public List<FlowNode> getNodes(Pager pager) {
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        StringBuilder sql = new StringBuilder("select t.* from wf_node t");
        boolean isFetchActor = paramMap.get("actorId") != null;
        boolean isFetchInstance = paramMap.get("instanceState") != null;
        if (isFetchActor) {
            sql.append(" left join wf_node_actor ta on ta.node_id = t.id ");
        }
        if (isFetchInstance) {
            sql.append(" left join wf_instance wfi on t.instance_id=wfi.id ");
        }
        sql.append(" where 1=1 ");
        Map<String, String> sorter = new HashMap<>();
        sorter.put(Common.ID, Common.ONE);
        pager.setSorter(Optional.ofNullable(pager.getSorter()).orElse(sorter));
        String query = getQueryBuilder().buildRaw(sql.toString(), pager, "");
        return super.list(FlowNode.class, query, paramMap);
    }

}
