package com.bocloud.fms.engine.handlers.impl;

import com.bocloud.fms.enums.FlowConstants;
import com.bocloud.fms.model.BasicDomainModel;
import com.bocloud.fms.engine.FlowEngine;
import com.bocloud.fms.engine.handlers.IHandler;
import com.bocloud.fms.engine.model.FlowNodeModel;
import com.bocloud.fms.entity.FlowInstance;
import com.bocloud.fms.entity.FlowNode;
import com.bocloud.fms.service.Execution;
import com.megatron.common.model.Param;
import com.megatron.common.model.RequestContext;
import com.megatron.common.model.Sign;
import com.megatron.common.utils.MapTools;
import lombok.extern.slf4j.Slf4j;

import java.util.ArrayList;
import java.util.List;

/**
 * 结束流程实例的处理器
 *
 * @author hufy
 */
@Slf4j
public class EndDefinitionHandler implements IHandler {

    /**
     * 结束当前流程实例
     */
    @Override
    public void handle(Execution execution) {

        FlowEngine engine = execution.getEngine();
        FlowInstance flowInstance = execution.getFlowInstance();

        List<Param> params = new ArrayList<>();
        params.add(new Param(MapTools.simpleMap("t.instanceId", flowInstance.getId()), Sign.EQ));
        params.add(new Param(MapTools.simpleMap("nodeState", FlowConstants.STATE_ACTIVE), Sign.EQ));

        List<BasicDomainModel<FlowNode, FlowNodeModel>> tasks = engine.query().getActiveNodes(params);
        for (BasicDomainModel<FlowNode, FlowNodeModel> task : tasks) {
            engine.node().complete(task.getEntity().getId(), new RequestContext(execution.getContext().getCatalog(), FlowConstants.AUTO, execution.getContext().getTenant(), null, null));
        }
        /**
         * 结束当前流程实例
         */
        engine.flowInstance().complete(flowInstance.getId());

    }
}
