package com.bocloud.fms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.internel.entity.Tenant;
import com.megatron.common.model.GeneralResult;
import com.megatron.common.model.Pager;
import com.megatron.common.utils.Common;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * soc服务租户内部访问
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年1月4日
 */
@Component
public class FmsSmsTenantService {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public FmsSmsTenantService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private static final Service SERVICE = Service.create(BoCloudService.SMS.getName());

    /**
     * 获取租户详情
     *
     * @param id 租户id
     * @return
     */
    public Tenant detail(Long id) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remote = serviceFactory.build(SERVICE, "/v1/tenants/" + id, HttpMethod.GET, headers, null);
        GeneralResult result = remote.invoke();
        if (result.isSuccess() && result.getData() != null) {
            return JSONObject.parseObject(JSONObject.toJSONString(result.getData()), Tenant.class);
        } else {
            return null;
        }
    }

    /**
     * 分页获取用户列表
     *
     * @param pager 分页对象
     * @return
     */
    public List<Tenant> list(Pager pager) {
        Map<String, Object> headers = new HashMap<>(4);
        headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS.name()), "/v1/tenants",
                HttpMethod.GET, headers, pager.toMap());
        GeneralResult<?> result = remoteService.invoke();
        return ((JSONObject) result.getData()).getJSONArray(Common.ROWS).toJavaList(Tenant.class);
    }


}
