package com.bocloud.fms.service.internal;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.cmp.boot.model.BoCloudService;
import com.bocloud.cmp.boot.model.BocloudToken;
import com.bocloud.internel.entity.User;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import com.megatron.framework.core.Service;
import com.megatron.framework.http.core.HttpMethod;
import com.megatron.framework.http.core.ServiceFactory;
import com.megatron.framework.http.model.RemoteService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author hufy
 */
@Slf4j
@Component
public class FmsSmsUserService implements InitializingBean {

    private final ServiceFactory serviceFactory;
    private final CurrentService currentService;

    @Autowired
    public FmsSmsUserService(ServiceFactory serviceFactory, CurrentService currentService) {
        this.serviceFactory = serviceFactory;
        this.currentService = currentService;
    }

    private final Map<String, Object> headers = new HashMap<>(4);

    private static final Service SERVICE = Service.create(BoCloudService.SMS.getName());


    /**
     * 获取用户详情
     *
     * @param id 用户id
     * @return
     */
    public GeneralResult<User> detail(Long id) {
        RemoteService remoteService = serviceFactory.build(SERVICE, "/v1/users/" + id, HttpMethod.GET,
                headers, null);
        GeneralResult result = remoteService.invoke();
        if (result.isFailed()) {
            return new GeneralResult<>(false, result.getMessage());
        }
        String data = JSONObject.toJSONString(result.getData());
        return new GeneralResult<>(true, JSONObject.parseObject(data, User.class), "success");
    }

    /**
     * 分页获取用户列表
     *
     * @param pager  分页对象
     * @return
     */
    public GeneralResult list(Pager pager) {
        RemoteService remoteService = serviceFactory.build(Service.create(BoCloudService.SMS.getName()), "/v1/users",
                HttpMethod.GET, headers, pager.toMap());
        GeneralResult<?> invoke = remoteService.invoke();
        return invoke;
    }

    /**
     * 根据id集合获取对应用户集合
     *
     * @param context 请求对象
     * @param ids     用户id集合
     * @return
     */
    public List<User> listByIds(RequestContext context, List<Long> ids) {
        List<Param> params = new ArrayList<>();
        params.add(new Param(
                MapTools.simpleMap("id",
                        JSONObject.toJSONString(ids).substring(1, JSONObject.toJSONString(ids).length() - 1)),
                Sign.IN));
        params.add(new Param(MapTools.simpleMap("status", "NORMAL"), Sign.EQ));
        GeneralResult result = list(new Pager(1, Integer.MAX_VALUE, params, null));
        return ((JSONObject) result.getData()).getJSONArray(Common.ROWS).toJavaList(User.class);
    }


    @Override
    public void afterPropertiesSet() throws Exception {
        this.headers.put(BocloudToken.INTERNAL_TOKEN, currentService.getToken());
    }

}
